@@ -447,8 +447,10 @@ bool AudioCDProtocol::getSectorsForRequest(struct cdrom_drive * drive, long & fi
 void AudioCDProtocol::get(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
-	if (!drive)
+	if (!drive) {
+		error(KIO::ERR_DOES_NOT_EXIST, url.path());
 		return;
+	}
 
 	if( d->fname.contains(i18n(CDDB_INFORMATION))){
 		uint choice = 1;
@@ -494,6 +496,7 @@ void AudioCDProtocol::get(const KUrl & url)
 
 	AudioCDEncoder *encoder = determineEncoder(d->fname);
 	if(!encoder){
+		error(KIO::ERR_DOES_NOT_EXIST, url.path());
 		cdda_close(drive);
 		return;
 	}
@@ -534,8 +537,25 @@ void AudioCDProtocol::get(const KUrl & url)
 void AudioCDProtocol::stat(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
-	if (!drive)
+
+	if (!drive && d->device.isEmpty()) {
+		// This is top level directory with CDROM devices
+		const mode_t _umask = ::umask(0);
+		::umask(_umask);
+		UDSEntry entry;
+		entry.insert(KIO::UDSEntry::UDS_NAME, url.fileName().replace(QLatin1Char( '/' ), QLatin1String("%2F")));
+		entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+		entry.insert(KIO::UDSEntry::UDS_ACCESS, (0666 & (~_umask)));
+		entry.insert(KIO::UDSEntry::UDS_SIZE, 2+encoders.count());
+		statEntry(entry);
+		finished();
 		return;
+	}
+
+	if (!drive) {
+		error(KIO::ERR_DOES_NOT_EXIST, url.path());
+		return;
+	}
 
 	const bool isFile = !d->fname.isEmpty();
 
@@ -549,6 +569,7 @@ void AudioCDProtocol::stat(const KUrl & url)
 		if (isFile && (trackNumber < 1 || trackNumber > d->tracks))
 		{
 			error(KIO::ERR_DOES_NOT_EXIST, url.path());
+			cdda_close(drive);
 			return;
 		}
 	}
@@ -610,9 +631,30 @@ void AudioCDProtocol::listDir(const KUrl & url)
 {
 	struct cdrom_drive * drive = initRequest(url);
 
-	// Some error checking before proceeding
-	if (!drive)
+	if (!drive && d->device.isEmpty()) {
+		// List CDROM devices
+		UDSEntry entry;
+		const QStringList &deviceNames = KCompactDisc::cdromDeviceNames();
+		foreach (const QString &deviceName, deviceNames) {
+			const QString &device = KCompactDisc::urlToDevice(KCompactDisc::cdromDeviceUrl(deviceName));
+			KUrl targetUrl = url;
+			targetUrl.addEncodedQueryItem("device", device.toUtf8());
+			app_dir(entry, device, 2+encoders.count());
+			entry.insert(KIO::UDSEntry::UDS_TARGET_URL, targetUrl.url());
+			entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, deviceName);
+			listEntry(entry, false);
+		}
+		totalSize(entry.count());
+		listEntry(entry, true);
+		finished();
 		return;
+	}
+
+	// Some error checking before proceeding
+	if (!drive) {
+		error(KIO::ERR_DOES_NOT_EXIST, url.path());
+		return;
+	}
 
 	if (d->which_dir == Unknown){
 		error(KIO::ERR_DOES_NOT_EXIST, url.path());
@@ -686,7 +728,7 @@ void AudioCDProtocol::listDir(const KUrl & url)
 	}
 
 	if (d->which_dir == SubDir || d->which_dir == Root || d->which_dir == EncoderDir) {
-		if (d->child_dir.isEmpty() || d->which_dir == Root || d->which_dir == EncoderDir)
+		if (d->child_dir.isEmpty())
 		{
 			// we are at the end of the hierarchy, list the tracks
 			for (uint trackNumber = 1; trackNumber <= d->tracks; trackNumber++)
@@ -711,8 +753,7 @@ void AudioCDProtocol::listDir(const KUrl & url)
 				}
 			}
 		}
-
-		if (!d->child_dir.isEmpty())
+		else
 		{
 			app_dir(entry, d->child_dir, 1);
 			listEntry(entry, false);
@@ -763,20 +804,12 @@ struct cdrom_drive *AudioCDProtocol::getDrive()
 {
 	const QByteArray device(QFile::encodeName(d->device));
 
+	if (device.isEmpty())
+		return 0;
+
 	struct cdrom_drive * drive = 0;
 
-	if (!device.isEmpty() && device != "/" )
-		drive = cdda_identify(device, CDDA_MESSAGE_PRINTIT, 0);
-	else
-	{
-		drive = cdda_find_a_cdrom(CDDA_MESSAGE_PRINTIT, 0);
-
-		if (0 == drive)
-		{
-			if (QFile(QFile::decodeName(KCompactDisc::defaultCdromDeviceUrl().url().toAscii())).exists())
-				drive = cdda_identify(KCompactDisc::defaultCdromDeviceUrl().url().toAscii(), CDDA_MESSAGE_PRINTIT, 0);
-		}
-	}
+	drive = cdda_identify(device, CDDA_MESSAGE_FORGETIT, 0);
 
 	if (0 == drive) {
 		kDebug(7117) << "Can't find an audio CD on: \"" << d->device << "\"";
@@ -842,7 +875,7 @@ void AudioCDProtocol::paranoiaRead(
 
 	paranoia_modeset(paranoia, paranoiaLevel);
 
-	cdda_verbose_set(drive, CDDA_MESSAGE_PRINTIT, CDDA_MESSAGE_PRINTIT);
+	cdda_verbose_set(drive, CDDA_MESSAGE_PRINTIT, CDDA_MESSAGE_FORGETIT);
 
 	paranoia_seek(paranoia, firstSector, SEEK_SET);
 
@@ -1020,10 +1053,7 @@ void AudioCDProtocol::loadSettings()
 	const KConfig *config = new KConfig(QLatin1String( "kcmaudiocdrc"), KConfig::NoGlobals );
         const KConfigGroup groupCDDA( config, "CDDA" );
 
-	if (!groupCDDA.readEntry("autosearch", true)) {
-		d->device = groupCDDA.readEntry("device", QString(KCompactDisc::defaultCdromDeviceUrl().url()));
-	}
-
+	d->device = QString(); // clear device
 	d->paranoiaLevel = 1; // enable paranoia error correction, but allow skipping
 
 	if (groupCDDA.readEntry("disable_paranoia", false)) {
